@@ -2,13 +2,14 @@ package DateTime::Format::Natural::Base;
 
 use strict;
 use warnings;
+use boolean qw(true false);
 
 use Date::Calc qw(Add_Delta_Days
                   Decode_Day_of_Week
                   Nth_Weekday_of_Month_Year
                   check_date check_time);
 
-our $VERSION = '1.13';
+our $VERSION = '1.17';
 
 use constant MORNING   => '08';
 use constant AFTERNOON => '14';
@@ -172,9 +173,11 @@ sub _daytime_in_the_morning
     $self->_add_trace;
     my ($hour) = @_;
     if ($self->_valid_time(hour => $hour)) {
-        $self->_set(hour => $hour);
-        $self->_set(minute => 0);
-        $self->_set(second => 0);
+        $self->_set(
+            hour   => $hour,
+            minute => 0,
+            second => 0,
+        );
     }
     $self->_set_modified(4);
 }
@@ -185,9 +188,11 @@ sub _daytime_in_the_afternoon
     $self->_add_trace;
     my ($hour) = @_;
     if ($self->_valid_time(hour => 12 + $hour)) {
-        $self->_set(hour => 12 + $hour);
-        $self->_set(minute => 0);
-        $self->_set(second => 0);
+        $self->_set(
+            hour   => 12 + $hour,
+            minute => 0,
+            second => 0,
+        );
     }
     $self->_set_modified(4);
 }
@@ -198,9 +203,11 @@ sub _daytime_in_the_evening
     $self->_add_trace;
     my ($hour) = @_;
     if ($self->_valid_time(hour => 12 + $hour)) {
-        $self->_set(hour => 12 + $hour);
-        $self->_set(minute => 0);
-        $self->_set(second => 0);
+        $self->_set(
+            hour   => 12 + $hour,
+            minute => 0,
+            second => 0,
+        );
     }
     $self->_set_modified(4);
 }
@@ -213,9 +220,11 @@ sub _daytime_morning
       ? $self->{Opts}{daytime}{morning}
       : MORNING;
     if ($self->_valid_time(hour => $hour)) {
-        $self->_set(hour => $hour);
-        $self->_set(minute => 0);
-        $self->_set(second => 0);
+        $self->_set(
+            hour   => $hour,
+            minute => 0,
+            second => 0,
+        );
     }
     $self->_set_modified(1);
 }
@@ -224,9 +233,11 @@ sub _daytime_noon
 {
     my $self = shift;
     $self->_add_trace;
-    $self->_set(hour => 12);
-    $self->_set(minute => 0);
-    $self->_set(second => 0);
+    $self->_set(
+        hour   => 12,
+        minute => 0,
+        second => 0,
+    );
     $self->_set_modified(1);
 }
 
@@ -238,9 +249,11 @@ sub _daytime_afternoon
       ? $self->{Opts}{daytime}{afternoon}
       : AFTERNOON;
     if ($self->_valid_time(hour => $hour)) {
-        $self->_set(hour => $hour);
-        $self->_set(minute => 0);
-        $self->_set(second => 0);
+        $self->_set(
+            hour   => $hour,
+            minute => 0,
+            second => 0,
+        );
     }
     $self->_set_modified(1);
 }
@@ -253,9 +266,11 @@ sub _daytime_evening
       ? $self->{Opts}{daytime}{evening}
       : EVENING;
     if ($self->_valid_time(hour => $hour)) {
-        $self->_set(hour => $hour);
-        $self->_set(minute => 0);
-        $self->_set(second => 0);
+        $self->_set(
+            hour   => $hour,
+            minute => 0,
+            second => 0,
+        );
     }
     $self->_set_modified(1);
 }
@@ -264,9 +279,11 @@ sub _daytime_midnight
 {
     my $self = shift;
     $self->_add_trace;
-    $self->_set(hour => 0);
-    $self->_set(minute => 0);
-    $self->_set(second => 0);
+    $self->_set(
+        hour   => 0,
+        minute => 0,
+        second => 0,
+    );
     $self->_set_modified(2);
 }
 
@@ -274,8 +291,10 @@ sub _hourtime_before_noon
 {
     my $self = shift;
     $self->_add_trace;
-    $self->_set(hour => 12);
-    $self->_set(minute => 0);
+    $self->_set(
+        hour   => 12,
+        minute => 0,
+    );
     $self->_subtract(hour => shift);
     $self->_set_modified(4);
 }
@@ -284,8 +303,10 @@ sub _hourtime_after_noon
 {
     my $self = shift;
     $self->_add_trace;
-    $self->_set(hour => 12);
-    $self->_set(minute => 0);
+    $self->_set(
+        hour   => 12,
+        minute => 0,
+    );
     $self->_add(hour => shift);
     $self->_set_modified(4);
 }
@@ -294,8 +315,10 @@ sub _hourtime_before_midnight
 {
     my $self = shift;
     $self->_add_trace;
-    $self->_set(hour => 0);
-    $self->_set(minute => 0);
+    $self->_set(
+        hour   => 0,
+        minute => 0,
+    );
     $self->_subtract(hour => shift);
     $self->_set_modified(4);
 }
@@ -304,8 +327,10 @@ sub _hourtime_after_midnight
 {
     my $self = shift;
     $self->_add_trace;
-    $self->_set(hour => 0);
-    $self->_set(minute => 0);
+    $self->_set(
+        hour   => 0,
+        minute => 0,
+    );
     $self->_add(hour => shift);
     $self->_set_modified(4);
 }
@@ -414,7 +439,7 @@ sub _last_day
     $self->_add_trace;
     my ($day) = @_;
     $self->_day_name(\$day);
-    my $days_diff = $self->{datetime}->wday + (7 - $self->{data}->{weekdays}->{$day});
+    my $days_diff = $self->_last_wday_diff($day);
     $self->_subtract(day => $days_diff);
     $self->_set_modified(2);
 }
@@ -424,7 +449,7 @@ sub _last_week_day
     my $self = shift;
     $self->_add_trace;
     my $day = ucfirst lc(shift);
-    my $days_diff = $self->{datetime}->wday + (7 - $self->{data}->{weekdays}->{$day});
+    my $days_diff = $self->_last_wday_diff($day);
     $self->_subtract(day => $days_diff);
     $self->_set_modified(3);
 }
@@ -434,7 +459,7 @@ sub _day_last_week
     my $self = shift;
     $self->_add_trace;
     my $day = ucfirst lc(shift);
-    my $days_diff = $self->{datetime}->wday + (7 - $self->{data}->{weekdays}->{$day});
+    my $days_diff = $self->_last_wday_diff($day);
     $self->_subtract(day => $days_diff);
     $self->_set_modified(3);
 }
@@ -500,7 +525,7 @@ sub _next_weekday
     $self->_add_trace;
     my ($day) = @_;
     $self->_day_name(\$day);
-    my $days_diff = (7 - $self->{datetime}->wday + Decode_Day_of_Week($day));
+    my $days_diff = $self->_next_wday_diff($day);
     $self->_add(day => $days_diff);
     $self->_set_modified(2);
 }
@@ -511,7 +536,7 @@ sub _weekday_next_week
     $self->_add_trace;
     my ($day) = @_;
     $self->_day_name(\$day);
-    my $days_diff = (7 - $self->{datetime}->wday + Decode_Day_of_Week($day));
+    my $days_diff = $self->_next_wday_diff($day);
     $self->_add(day => $days_diff);
     $self->_set_modified(3);
 }
@@ -633,14 +658,26 @@ sub _count_weekday_this_month
     $self->_day_name(\$day);
     $self->_month_name(\$month);
     my $year;
-    ($year, $month, $day) =
-      Nth_Weekday_of_Month_Year($self->{datetime}->year,
-                                $self->{data}->{months}->{$month},
-                                $self->{data}->{weekdays}->{$day},
-                                $count);
-    $self->_set(year => $year);
-    $self->_set(month => $month);
-    $self->_set(day => $day);
+    eval {
+        ($year, $month, $day) =
+          Nth_Weekday_of_Month_Year($self->{datetime}->year,
+                                    $self->{data}->{months}->{$month},
+                                    $self->{data}->{weekdays}->{$day},
+                                    $count);
+    };
+    if (!$@ and defined $year && defined $month && defined $day
+        and $self->_valid_date(year => $year, month => $month, day => $day))
+    {
+        $self->_set(
+            year  => $year,
+            month => $month,
+            day   => $day,
+        );
+    }
+    else {
+        $self->_set_failure;
+        $self->_set_error("(date is not valid)");
+    }
     $self->_set_modified(4);
 }
 
@@ -687,14 +724,18 @@ sub _at_am
     if ($time =~ /:/) {
         my ($hour, $minute) = split /:/, $time;
         if ($self->_valid_time(hour => $hour, minute => $minute)) {
-            $self->_set(hour => $hour);
-            $self->_set(minute => $minute);
+            $self->_set(
+                hour   => $hour,
+                minute => $minute,
+            );
         }
     }
     else {
         if ($self->_valid_time(hour => $time)) {
-            $self->_set(hour => $time);
-            $self->_set(minute => 0);
+            $self->_set(
+                hour   => $time,
+                minute => 0,
+            );
         }
     }
     $self->_set_modified(2);
@@ -708,14 +749,18 @@ sub _at_pm
     if ($time =~ /:/) {
         my ($hour, $minute) = split /:/, $time;
         if ($self->_valid_time(hour => 12 + $hour, minute => $minute)) {
-            $self->_set(hour => 12 + $hour);
-            $self->_set(minute => $minute);
+            $self->_set(
+                hour   => 12 + $hour,
+                minute => $minute,
+            );
         }
     }
     else {
         if ($self->_valid_time(hour => 12 + $time)) {
-            $self->_set(hour => 12 + $time);
-            $self->_set(minute => 0);
+            $self->_set(
+                hour   => 12 + $time,
+                minute => 0,
+            );
         }
     }
     $self->_set_modified(2);
@@ -729,14 +774,18 @@ sub _time
     if ($time =~ /:/) {
         my ($hour, $minute) = split /:/, $time;
         if ($self->_valid_time(hour => $hour, minute => $minute)) {
-            $self->_set(hour => $hour);
-            $self->_set(minute => $minute);
+            $self->_set(
+                hour   => $hour,
+                minute => $minute,
+            );
         }
     }
     else {
         if ($self->_valid_time(hour => $time)) {
-            $self->_set(hour => $time);
-            $self->_set(minute => 0);
+            $self->_set(
+                hour   => $time,
+                minute => 0,
+            );
         }
     }
     $self->_set_modified(1);
@@ -749,9 +798,11 @@ sub _time_full
     my ($time) = @_;
     my ($hour, $minute, $second) = split /:/, $time;
     if ($self->_valid_time(hour => $hour, minute => $minute, second => $second)) {
-        $self->_set(hour => $hour);
-        $self->_set(minute => $minute);
-        $self->_set(second => $second);
+        $self->_set(
+            hour   => $hour,
+            minute => $minute,
+            second => $second,
+        );
     }
     $self->_set_modified(1);
 }
@@ -770,9 +821,11 @@ sub _count_yearday
     my ($day) = @_;
     my ($year, $month);
     ($year, $month, $day) = Add_Delta_Days($self->{datetime}->year, 1, 1, $day - 1);
-    $self->_set(day => $day);
-    $self->{datetime}->set_month($month);
-    $self->{datetime}->set_year($year);
+    $self->_set(
+        year  => $year,
+        month => $month,
+        day   => $day,
+    );
     $self->_set_modified(2);
 }
 
@@ -782,19 +835,44 @@ sub _count_weekday
     $self->_add_trace;
     my ($count, $weekday) = @_;
     $weekday = ucfirst lc $weekday;
-    my ($year, $month, $day) =
-      Nth_Weekday_of_Month_Year($self->{datetime}->year,
-                                $self->{datetime}->month,
-                                $self->{data}->{weekdays}->{$weekday},
-                                $count);
-    if ($self->_valid_date(day => $day, month => $month, year => $year)) {
-        $self->_set(day => $day);
-        $self->_set(month => $month);
-        $self->{datetime}->set_year($year);
+    my ($year, $month, $day);
+    eval {
+        ($year, $month, $day) =
+          Nth_Weekday_of_Month_Year($self->{datetime}->year,
+                                    $self->{datetime}->month,
+                                    $self->{data}->{weekdays}->{$weekday},
+                                    $count);
+    };
+    if (!$@ and defined $year && defined $month && defined $day
+        and $self->_valid_date(year => $year, month => $month, day => $day))
+    {
+        $self->_set(
+            year  => $year,
+            month => $month,
+            day   => $day,
+        );
+    }
+    else {
+        $self->_set_failure;
+        $self->_set_error("(date is not valid)");
     }
     $self->_set_modified(2);
 }
 
+sub _day_month_year
+{
+    my $self = shift;
+    $self->_add_trace;
+    my ($day, $month, $year) = @_;
+    $self->_month_name(\$month);
+    $self->_set(
+        year  => $year,
+        month => $self->_month_num($month),
+        day   => $day,
+    );
+    $self->_set_modified(3);
+}
+
 sub _day_name
 {
     my $self = shift;
@@ -826,67 +904,106 @@ sub _month_num
     return $self->{data}->{months}->{$month};
 }
 
+sub _last_wday_diff
+{
+    my $self = shift;
+    my ($day) = @_;
+    return $self->{datetime}->wday + (7 - $self->{data}->{weekdays}->{$day});
+}
+
+sub _next_wday_diff
+{
+    my $self = shift;
+    my ($day) = @_;
+    return (7 - $self->{datetime}->wday + Decode_Day_of_Week($day));
+}
+
 sub _add
 {
     my ($self, $unit, $value) = @_;
 
-    $self->{modified}{$unit}++;
-
     $unit .= 's' unless $unit =~ /s$/;
     $self->{datetime}->add($unit => $value);
+
+    chop $unit;
+    $self->{modified}{$unit}++;
 }
 
 sub _subtract
 {
     my ($self, $unit, $value) = @_;
 
-    $self->{modified}{$unit}++;
-
     $unit .= 's' unless $unit =~ /s$/;
     $self->{datetime}->subtract($unit => $value);
+
+    chop $unit;
+    $self->{modified}{$unit}++;
 }
 
 sub _set
 {
-    my ($self, $unit, $value) = @_;
+    my ($self, %values) = @_;
 
-    $self->{modified}{$unit}++;
+    my @units = qw(
+        year
+        month
+        day
+        hour
+        minute
+        second
+    );
 
-    my $setter = 'set_' . $unit;
-    $self->{datetime}->$setter($value);
+    foreach my $unit (@units) {
+        if (exists $values{$unit}) {
+            my $setter = 'set_' . $unit;
+            $self->{datetime}->$setter($values{$unit});
+            $self->{modified}{$unit}++;
+        }
+    }
 }
 
 sub _valid_date
 {
-    my ($self, $type, $value) = @_;
+    my ($self, %values) = @_;
 
     my %set = map { $_ => $self->{datetime}->$_ } qw(year month day);
-    $set{$type} = $value;
+
+    while (my ($unit, $value) = each %values) {
+        $set{$unit} = $value;
+    }
 
     if (check_date($set{year}, $set{month}, $set{day})) {
-        return 1;
+        return true;
     }
     else {
         $self->_set_failure;
-        $self->_set_error("('$value' is not a valid $type)");
-        return 0;
+        $self->_set_error("(date is not valid)");
+        return false;
     }
 }
 
 sub _valid_time
 {
-    my ($self, $type, $value) = @_;
+    my ($self, %values) = @_;
 
-    my %set = map { $_ => $self->{datetime}->$_ } qw(hour min sec);
-    $set{$type} = $value;
+    my %abbrev = (
+        second => 'sec',
+        minute => 'min',
+        hour   => 'hour',
+    );
+    my %set = map { $_ => $self->{datetime}->$_ } values %abbrev;
+
+    while (my ($unit, $value) = each %values) {
+        $set{$abbrev{$unit}} = $value;
+    }
 
     if (check_time($set{hour}, $set{min}, $set{sec})) {
-        return 1;
+        return true;
     }
     else {
         $self->_set_failure;
-        $self->_set_error("('$value' is not a valid $type)");
-        return 0;
+        $self->_set_error("(time is not valid)");
+        return false;
     }
 }
 
